#pragma once

#ifndef __OPENCV__
  #error 666: do not include this file manually include only 'opencv.bi' !
#endif


'#include "opencv_core.bi"
'#include "opencv_features2d.bi"

extern "C"

' ##################################################
' # Camera Calibration, Pose Estimation and Stereo #
' ##################################################

type CvPOSITObject as any

' Allocates and initializes CvPOSITObject structure before doing cvPOSIT
declare function cvCreatePOSITObject( points as CvPoint3D32f ptr, point_count as long ) as CvPOSITObject ptr


' Runs POSIT (POSe from ITeration) algorithm for determining 3d position 
' of an object given its model and projection in a weak-perspective case
declare sub      cvPOSIT( posit_object as CvPOSITObject ptr, image_points as CvPoint2D32f ptr, _
                          focal_length as double, criteria as CvTermCriteria, _
                          rotation_matrix as single ptr, translation_vector as single ptr)

' Releases CvPOSITObject structure
declare sub      cvReleasePOSITObject( posit_object as CvPOSITObject ptr ptr )

' updates the number of RANSAC iterations
declare function cvRANSACUpdateNumIters( p as double, err_prob as double, _
                                         model_points as long, max_iters as long ) as long

declare sub      cvConvertPointsHomogeneous( src as const CvMat ptr, dst as CvMat ptr )

' Calculates fundamental matrix given a set of corresponding points
#define CV_FM_7POINT 1
#define CV_FM_8POINT 2

#define CV_LMEDS 4
#define CV_RANSAC 8

#define CV_FM_LMEDS_ONLY  CV_LMEDS
#define CV_FM_RANSAC_ONLY CV_RANSAC
#define CV_FM_LMEDS CV_LMEDS
#define CV_FM_RANSAC CV_RANSAC

enum
  CV_ITERATIVE = 0,
  CV_EPNP      = 1, ' F.Moreno-Noguer, V.Lepetit and P.Fua "EPnP: Efficient Perspective-n-Point Camera Pose Estimation"
  CV_P3P       = 2  ' X.S. Gao, X.-R. Hou, J. Tang, H.-F. Chang; "Complete Solution Classification for the Perspective-Three-Point Problem"
end enum

declare function cvFindFundamentalMat( points1 as const CvMat ptr, _
                                       points2 as const CvMat ptr, _
                                       fundamental_matrix as CvMat ptr, _
                                       method as long CV_DEFAULT(CV_FM_RANSAC), _
                                       param1 as double CV_DEFAULT(3.), _
                                       param2 as double CV_DEFAULT(0.99), _
                                       status as CvMat ptr CV_DEFAULT(NULL) ) as long

' For each input point on one of images computes parameters 
' of the corresponding epipolar line on the other image
declare sub      cvComputeCorrespondEpilines( points as const CvMat ptr, _
                                              which_image as long, _
                                              fundamental_matrix as const CvMat ptr, _
                                              correspondent_lines as CvMat ptr )

' Triangulation functions
declare sub      cvTriangulatePoints( projMatr1 as CvMat ptr, _
                                      projMatr2 as CvMat ptr, _
                                      projPoints1 as CvMat ptr, _
                                      projPoints2 as CvMat ptr, _
                                      points4D as CvMat ptr )

declare sub      cvCorrectMatches( F           as CvMat ptr, _
                                   points1     as CvMat ptr, _
                                   points2     as CvMat ptr, _
                                   new_points1 as CvMat ptr, _
                                   new_points2 as CvMat ptr )


' Computes the optimal new camera matrix according to the free scaling parameter alpha:
' alpha=0 - only valid pixels will be retained in the undistorted image
' alpha=1 - all the source image pixels will be retained in the undistorted image
declare sub      cvGetOptimalNewCameraMatrix( camera_matrix as const CvMat ptr, _
                                              dist_coeffs   as const CvMat ptr, _
                                              image_size    as CvSize, _
                                              alpha_        as double, _
                                              new_camera_matrix as CvMat ptr, _
                                              new_imag_size as CvSize CV_DEFAULT(cv_Size(0,0)), _
                                              valid_pixel_ROI as CvRect ptr CV_DEFAULT(NULL), _
                                              center_principal_point as long CV_DEFAULT(0) )

' Converts rotation vector to rotation matrix or vice versa
declare function cvRodrigues2( src as const CvMat ptr, _
                               dst as CvMat ptr, _
                               jacobian as CvMat ptr CV_DEFAULT(0) ) as long

' Finds perspective transformation between the object plane and image (view) plane
declare function cvFindHomography( src_points as const CvMat ptr, _
                                   dst_points as const CvMat ptr, _
                                   homography as CvMat ptr, _
                                   method as long CV_DEFAULT(0), _
                                   ransacReprojThreshold as double CV_DEFAULT(3), _
                                   mask as CvMat ptr CV_DEFAULT(0) ) as long

' Computes RQ decomposition for 3x3 matrices
declare sub      cvRQDecomp3x3( matrixM as const CvMat ptr, _
                                matrixR as CvMat ptr, _
                                matrixQ as CvMat ptr, _
                                matrixQx as CvMat ptr CV_DEFAULT(NULL), _
                                matrixQy as CvMat ptr CV_DEFAULT(NULL), _
                                matrixQz as CvMat ptr CV_DEFAULT(NULL), _
                                eulerAngles as CvPoint3D64f ptr CV_DEFAULT(NULL) )

' Computes projection matrix decomposition
declare sub      cvDecomposeProjectionMatrix( projMatr as const CvMat ptr, _
                                              calibMatr as CvMat ptr, _
                                              rotMatr   as CvMat ptr, _
                                              posVect   as CvMat ptr, _
                                              rotMatrX  as CvMat ptr CV_DEFAULT(NULL), _
                                              rotMatrY  as CvMat ptr CV_DEFAULT(NULL), _
                                              rotMatrZ  as CvMat ptr CV_DEFAULT(NULL), _
                                              eulerAngles as CvPoint3D64f ptr CV_DEFAULT(NULL) )

' Computes d(AB)/dA and d(AB)/dB 
declare sub      cvCalcMatMulDeriv( A as const CvMat ptr, B as const CvMat ptr, dABdA as CvMat ptr, dABdB as CvMat ptr )

' Computes r3 = rodrigues(rodrigues(r2)*rodrigues(r1)), t3 = rodrigues(r2)*t1 + t2 and the respective derivatives 
declare sub      cvComposeRT( rvec1 as const CvMat ptr, _
                              tvec1 as const CvMat ptr, _
                              rvec2 as const CvMat ptr, _
                              tvec2 as const CvMat ptr, _
                              rvec3 as CvMat ptr, _
                              tvec3 as CvMat ptr, _
                              dr3dr1 as CvMat ptr CV_DEFAULT(NULL), _
                              dr3dt1 as CvMat ptr CV_DEFAULT(NULL), _
                              dr3dr2 as CvMat ptr CV_DEFAULT(NULL), _
                              dr3dt2 as CvMat ptr CV_DEFAULT(NULL), _
                              dt3dr1 as CvMat ptr CV_DEFAULT(NULL), _
                              dt3dt1 as CvMat ptr CV_DEFAULT(NULL), _
                              dt3dr2 as CvMat ptr CV_DEFAULT(NULL), _
                              dt3dt2 as CvMat ptr CV_DEFAULT(NULL) )

' Projects object points to the view plane using
' the specified extrinsic and intrinsic camera parameters
declare sub      cvProjectPoints2( object_points as const CvMat ptr, _
                                   rotation_vector as const CvMat ptr, _
                                   translation_vector as const CvMat ptr, _
                                   camera_matrix as const CvMat ptr, _
                                   distortion_coeffs as const CvMat ptr, _
                                   image_points as CvMat ptr, _
                                   dpdrot as CvMat ptr CV_DEFAULT(NULL), _
                                   dpdt   as CvMat ptr CV_DEFAULT(NULL), _
                                   dpdf   as CvMat ptr CV_DEFAULT(NULL), _
                                   dpdc   as CvMat ptr CV_DEFAULT(NULL), _
                                   dpddist as CvMat ptr CV_DEFAULT(NULL), _
                                   aspect_ratio as double CV_DEFAULT(0) )

' Finds extrinsic camera parameters from a few known corresponding point pairs and intrinsic parameters
declare sub      cvFindExtrinsicCameraParams2( object_points as const CvMat ptr, _
                                               image_points as const CvMat ptr, _
                                               camera_matrix as const CvMat ptr, _
                                               distortion_coeffs as const CvMat ptr, _
                                               rotation_vector as CvMat ptr, _
                                               translation_vector as CvMat ptr, _
                                               use_extrinsic_guess as long CV_DEFAULT(0) )

' Computes initial estimate of the intrinsic camera parameters
' in case of planar calibration target (e.g. chessboard)
declare sub      cvInitIntrinsicParams2D( object_points as const CvMat ptr, _
                                          image_points as const CvMat ptr, _
                                          npoints as const CvMat ptr, _
                                          image_size as CvSize, _
                                          camera_matrix as CvMat ptr, _
                                          aspect_ratio as double CV_DEFAULT(1.) )

#define CV_CALIB_CB_ADAPTIVE_THRESH  1
#define CV_CALIB_CB_NORMALIZE_IMAGE  2
#define CV_CALIB_CB_FILTER_QUADS     4
#define CV_CALIB_CB_FAST_CHECK       8

' Performs a fast check if a chessboard is in the input image. 
' This is a workaround to a problem of cvFindChessboardCorners being slow on images with no chessboard
' - src: input image
' - size: chessboard size
' Returns 1 if a chessboard can be in this image and findChessboardCorners should be called,
' 0 if there is no chessboard, -1 in case of error
declare function cvCheckChessboard( src as IplImage ptr, size as CvSize ) as long

' Detects corners on a chessboard calibration pattern
declare function cvFindChessboardCorners( image as const any ptr, _
                                          pattern_size as CvSize, _
                                          corners as CvPoint2D32f ptr, _
                                          corner_count as long ptr CV_DEFAULT(NULL), _
                                          flags as long CV_DEFAULT(CV_CALIB_CB_ADAPTIVE_THRESH + CV_CALIB_CB_NORMALIZE_IMAGE) ) as long

' Draws individual chessboard corners or the whole chessboard detected
declare sub      cvDrawChessboardCorners( image as CvArr ptr, _
                                          pattern_size as CvSize, _
                                          corners as CvPoint2D32f ptr, _
                                          count as long, _
                                          pattern_was_found as long )

#define CV_CALIB_USE_INTRINSIC_GUESS  1
#define CV_CALIB_FIX_ASPECT_RATIO     2
#define CV_CALIB_FIX_PRINCIPAL_POINT  4
#define CV_CALIB_ZERO_TANGENT_DIST    8
#define CV_CALIB_FIX_FOCAL_LENGTH 16
#define CV_CALIB_FIX_K1  32
#define CV_CALIB_FIX_K2  64
#define CV_CALIB_FIX_K3  128
#define CV_CALIB_FIX_K4  2048
#define CV_CALIB_FIX_K5  4096
#define CV_CALIB_FIX_K6  8192
#define CV_CALIB_RATIONAL_MODEL 16384

' Finds intrinsic and extrinsic camera parameters
' from a few views of known calibration pattern
declare function cvCalibrateCamera2( object_points as const CvMat ptr, _
                                     image_points as const CvMat ptr, _
                                     point_counts as const CvMat ptr, _
                                     image_size as CvSize, _
                                     camera_matrix as CvMat ptr, _
                                     distortion_coeffs as CvMat ptr, _
                                     rotation_vectors as CvMat ptr CV_DEFAULT(NULL), _
                                     translation_vectors as CvMat ptr CV_DEFAULT(NULL), _
                                     flags as long CV_DEFAULT(0), _
                                     term_crit as CvTermCriteria CV_DEFAULT( _
                                     cv_TermCriteria(CV_TERMCRIT_ITER + CV_TERMCRIT_EPS,30, DBL_EPSILON) ) _
                                     ) as double

' Computes various useful characteristics of the camera 
' from the data computed by cvCalibrateCamera2
declare sub      cvCalibrationMatrixValues( camera_matrix as const CvMat ptr, _
                                            image_size as CvSize, _
                                            aperture_width as double CV_DEFAULT(0), _
                                            aperture_height as double CV_DEFAULT(0), _
                                            fovx as double ptr CV_DEFAULT(NULL), _
                                            fovy as double ptr CV_DEFAULT(NULL), _
                                            focal_length as double ptr CV_DEFAULT(NULL), _
                                            principal_point as  CvPoint2D64f ptr CV_DEFAULT(NULL), _
                                            pixel_aspect_ratio as double ptr CV_DEFAULT(NULL) )

#define CV_CALIB_FIX_INTRINSIC  256
#define CV_CALIB_SAME_FOCAL_LENGTH 512

' Computes the transformation from one camera coordinate system 
' to another one from a few correspondent views of the same calibration target.
' Optionally, calibrates  both cameras
declare function cvStereoCalibrate( object_points as const CvMat ptr, _
                                    image_points1 as const CvMat ptr, _
                                    image_points2 as const CvMat ptr, _
                                    npoints as const CvMat ptr, _
                                    camera_matrix1 as CvMat ptr, _
                                    dist_coeffs1 as CvMat ptr, _
                                    camera_matrix2 as CvMat ptr, _
                                    dist_coeffs2 as CvMat ptr, _
                                    image_size as CvSize, _
                                    R as CvMat ptr, _
                                    T as CvMat ptr, _
                                    E as CvMat ptr CV_DEFAULT(NULL), _
                                    F as CvMat ptr CV_DEFAULT(NULL), _
                                    term_crit as CvTermCriteria CV_DEFAULT(cv_TermCriteria( _
                                    CV_TERMCRIT_ITER+CV_TERMCRIT_EPS,30,1e-6)), _
                                    flags as long CV_DEFAULT(CV_CALIB_FIX_INTRINSIC) ) as double

#define CV_CALIB_ZERO_DISPARITY 1024

' Computes 3D rotations (+ optional shift) for each camera coordinate system 
' to make both views parallel (=> to make all the epipolar lines horizontal or vertical)
declare sub      cvStereoRectify( camera_matrix1 as const CvMat ptr, _
                                  camera_matrix2 as const CvMat ptr, _
                                  dist_coeffs1 as const CvMat ptr, _
                                  dist_coeffs2 as const CvMat ptr, _
                                  image_size as CvSize, _
                                  R as const CvMat ptr, _
                                  T as const CvMat ptr, _
                                  R1 as CvMat ptr, _
                                  R2 as CvMat ptr, _
                                  P1 as CvMat ptr, _
                                  P2 as CvMat ptr, _
                                  Q as CvMat ptr CV_DEFAULT(NULL), _
                                  flags as long CV_DEFAULT(CV_CALIB_ZERO_DISPARITY), _
                                  alpha_ as double CV_DEFAULT(-1), _
                                  new_image_size as CvSize CV_DEFAULT(cv_Size(0,0)), _
                                  valid_pix_ROI1 as CvRect ptr CV_DEFAULT(0), _
                                  valid_pix_ROI2 as CvRect ptr CV_DEFAULT(0) )

' Computes rectification transformations for uncalibrated pair of images 
' using a set of point correspondences
declare function cvStereoRectifyUncalibrated( points1 as const CvMat ptr, _
                                              points2 as const CvMat ptr, _
                                              F as const CvMat ptr, _
                                              img_size as CvSize, _
                                              H1 as CvMat ptr, _
                                              H2 as CvMat ptr, _
                                              threshold as double CV_DEFAULT(5) ) as long

' stereo correspondence parameters and functions
#define CV_STEREO_BM_NORMALIZED_RESPONSE  0
#define CV_STEREO_BM_XSOBEL               1

' Block matching algorithm structure
type CvStereoBMState
  ' pre-filtering (normalization of input images)
  as long preFilterType ' =CV_STEREO_BM_NORMALIZED_RESPONSE now
  as long preFilterSize ' averaging window size: ~5x5..21x21
  as long preFilterCap  ' the output of pre-filtering is clipped by [-preFilterCap,preFilterCap]

  ' correspondence using Sum of Absolute Difference (SAD)
  as long SADWindowSize       ' ~5x5..21x21
  as long minDisparity        ' minimum disparity (can be negative)
  as long numberOfDisparities ' maximum disparity - minimum disparity (> 0)

  ' post-filtering
  as long textureThreshold ' the disparity is only computed for pixels
                           ' with textured enough neighborhood
  as long uniquenessRatio  ' accept the computed disparity d* only if
                           ' SAD(d) >= SAD(d*)*(1 + uniquenessRatio/100.)
                           ' for any d != d*+/-1 within the search range.
  as long speckleWindowSize ' disparity variation window
  as long speckleRange      ' acceptable range of variation in window

  as long trySmallerWindows ' if 1, the results may be more accurate,
                            ' at the expense of slower processing
  as CvRect roi1, roi2
  as long disp12MaxDiff

  ' temporary buffers
  as CvMat ptr preFilteredImg0
  as CvMat ptr preFilteredImg1
  as CvMat ptr slidingSumBuf
  as CvMat ptr cost
  as CvMat ptr disp
end type

#define CV_STEREO_BM_BASIC 0
#define CV_STEREO_BM_FISH_EYE 1
#define CV_STEREO_BM_NARROW 2

declare function cvCreateStereoBMState( preset_ as long CV_DEFAULT(CV_STEREO_BM_BASIC), _
                                        numberOfDisparities as long CV_DEFAULT(0) ) as CvStereoBMState ptr

declare sub      cvReleaseStereoBMState( state as CvStereoBMState ptr ptr )

declare sub      cvFindStereoCorrespondenceBM( left_     as const CvArr ptr, _
                                               right_    as const CvArr ptr, _
                                               disparity as CvArr ptr, _
                                               state     as CvStereoBMState ptr )

declare function cvGetValidDisparityROI( roi1 as CvRect, _
                                         roi2 as CvRect, _
                                         minDisparity as long, _
                                         numberOfDisparities as long, _
                                         SADWindowSize as long ) as CvRect

declare sub      cvValidateDisparity( disparity           as CvArr ptr, _
                                      cost                as const CvArr ptr, _
                                      minDisparity        as long, _
                                      numberOfDisparities as long, _
                                      disp12MaxDiff       as long CV_DEFAULT(1) )

' Reprojects the computed disparity image to the 3D space using the specified 4x4 matrix
declare sub      cvReprojectImageTo3D( disparityImage as const CvArr ptr, _
                                       _3dImage as CvArr ptr, _
                                       Q as const CvMat ptr, _
                                       handleMissingValues as long CV_DEFAULT(0) )
end extern ' "C"
