#pragma once

#ifndef __OPENCV__
  #error 666: do not include this file manually include only 'opencv.bi' !
#endif

'#include "opencv_core.bi"
'#include "opencv_imgproc.bi"


extern "C"

' ###################
' # Motion Analysis #
' ###################

#define  CV_LKFLOW_PYR_A_READY       1
#define  CV_LKFLOW_PYR_B_READY       2
#define  CV_LKFLOW_INITIAL_GUESSES   4
#define  CV_LKFLOW_GET_MIN_EIGENVALS 8

' It is Lucas & Kanade method, modified to use pyramids.
' Also it does several iterations to get optical flow for every point at every pyramid level.
' Calculates optical flow between two images for certain set of points 
' (i.e. it is a "sparse" optical flow, which is opposite to the previous 3 methods)
declare sub cvCalcOpticalFlowPyrLK( prev as const CvArr ptr, _
                                    curr as const CvArr ptr, _
                                    prev_pyr as CvArr ptr, _
                                    curr_pyr as CvArr ptr, _
                                    prev_features as const CvPoint2D32f ptr, _
                                    curr_features as CvPoint2D32f ptr, _
                                    count as long, _
                                    win_size as CvSize, _
                                    level as long, _
                                    status as byte ptr, _
                                    track_error as single ptr, _
                                    criteria as CvTermCriteria, _
                                    flags as long )


' Modification of a previous sparse optical flow algorithm to calculate affine flow
declare sub cvCalcAffineFlowPyrLK( prev as const CvArr ptr, _
                                   curr as const CvArr ptr, _
                                   prev_pyr as CvArr ptr, _
                                   curr_pyr as CvArr ptr, _
                                   prev_features as const CvPoint2D32f ptr, _
                                   curr_features as CvPoint2D32f ptr, _
                                   matrices as single ptr, _
                                   count as long, _
                                   win_size as CvSize, _
                                   level as long, _
                                   status as byte ptr, _
                                   track_error as single ptr, _
                                   criteria as CvTermCriteria, _
                                   flags as long )

' Estimate rigid transformation between 2 images or 2 point sets
declare function cvEstimateRigidTransform( A as const CvArr ptr, _
                                           B as const CvArr ptr, _
                                           M as CvMat ptr, _
                                           full_affine as long) as long

' Estimate optical flow for each pixel using the two-frame G. Farneback algorithm
declare sub cvCalcOpticalFlowFarneback( prev as const CvArr ptr, _
                                        next_ as const CvArr ptr, _
                                        flow as CvArr ptr, _
                                        pyr_scale as double, _
                                        levels as long, _
                                        winsize as long, _
                                        iterations as long, _
                                        poly_n as long, _
                                        poly_sigma as double, _
                                        flags as long )
' ####################
' # motion templates #
' ####################

' All the motion template functions work only with single channel images.
' Silhouette image must have depth IPL_DEPTH_8U or IPL_DEPTH_8S
' Motion history image must have depth IPL_DEPTH_32F
' Gradient mask - IPL_DEPTH_8U or IPL_DEPTH_8S
' Motion orientation image - IPL_DEPTH_32F
' Segmentation mask - IPL_DEPTH_32F
' All the angles are in degrees, all the times are in milliseconds

' Updates motion history image given motion silhouette
declare sub cvUpdateMotionHistory( silhouette as const CvArr ptr, _
                                   mhi as CvArr ptr, _
                                   timestamp as double, _
                                   duration as double)

' Calculates gradient of the motion history image 
' and fills a mask indicating where the gradient is valid
declare sub cvCalcMotionGradient( mhi as const CvArr ptr, _
                                  mask as CvArr ptr, _
                                  orientation as CvArr ptr, _
                                  delta1 as double, _
                                  delta2 as double, _
                                  aperture_size as long CV_DEFAULT(3) )

' Calculates average motion direction within a selected motion region
' (region can be selected by setting ROIs and/or by composing 
' a valid gradient mask with the region mask)
declare function cvCalcGlobalOrientation( orientation as const CvArr ptr, _
                                          mask as const CvArr ptr, _
                                          mhi as const CvArr ptr, _
                                          timestamp as double, _
                                          duration as double ) as double

' Splits a motion history image into a few parts 
' corresponding to separate independent motions (e.g. left hand, right hand)
declare function cvSegmentMotion( mhi as const CvArr ptr, _
                                  seg_mask as CvArr ptr, _
                                  storage as CvMemStorage ptr, _
                                  timestamp as double, _
                                  seg_thresh as double) as CvSeq ptr

' ############
' # Tracking #
' ############

' Implements CAMSHIFT algorithm 
' determines object position, size and orientation 
' from the object histogram back project (extension of meanshift)
declare function cvCamShift( prob_image as const CvArr ptr, window_ as CvRect, _
                             criteria as CvTermCriteria, comp as CvConnectedComp ptr, _
                             box as CvBox2D ptr CV_DEFAULT(NULL) ) as long

' Implements MeanShift algorithm 
' determines object position from the object histogram back project
declare function cvMeanShift( prob_image as const CvArr ptr, window_ as CvRect, _
                              criteria as CvTermCriteria, comp as CvConnectedComp ptr ) as long 


' standard Kalman filter (in G. Welch' and G. Bishop's notation):
' x(k)=A*x(k-1)+B*u(k)+w(k)  p(w)~N(0,Q)
' z(k)=H*x(k)+v(k),   p(v)~N(0,R)
type CvKalman
  as long MP                     ' number of measurement vector dimensions
  as long DP                     ' number of state vector dimensions
  as long CP                     ' number of control vector dimensions

  ' backward compatibility fields
#if 1
  as single ptr PosterState         ' =state_pre->data.fl
  as single ptr PriorState          ' =state_post->data.fl
  as single ptr DynamMatr           ' =transition_matrix->data.fl
  as single ptr MeasurementMatr     ' =measurement_matrix->data.fl
  as single ptr MNCovariance        ' =measurement_noise_cov->data.fl
  as single ptr PNCovariance        ' =process_noise_cov->data.fl
  as single ptr KalmGainMatr        ' =gain->data.fl
  as single ptr PriorErrorCovariance ' =error_cov_pre->data.fl
  as single ptr PosterErrorCovariance ' =error_cov_post->data.fl
  as single ptr Temp1_               ' renamed: Temp1 temp1->data.fl
  as single ptr Temp2_               ' renamed: Temp1temp2->data.fl
#endif

  as CvMat ptr state_pre          ' predicted state (x'(k)): x(k)=A*x(k-1)+B*u(k)
  as CvMat ptr state_post          ' corrected state (x(k)): x(k)=x'(k)+K(k)*(z(k)-H*x'(k))
  as CvMat ptr transition_matrix   ' state transition matrix (A)
  as CvMat ptr control_matrix      ' control matrix (B) (it is not used if there is no control)
  as CvMat ptr measurement_matrix  ' measurement matrix (H)
  as CvMat ptr process_noise_cov   ' process noise covariance matrix (Q)
  as CvMat ptr measurement_noise_cov ' measurement noise covariance matrix (R)
  as CvMat ptr error_cov_pre       ' priori error estimate covariance matrix (P'(k)): P'(k)=A*P(k-1)*At + Q)
  as CvMat ptr gain                ' Kalman gain matrix (K(k)): K(k)=P'(k)*Ht*inv(H*P'(k)*Ht+R)
  as CvMat ptr error_cov_post      ' posteriori error estimate covariance matrix (P(k)): P(k)=(I-K(k)*H)*P'(k)
  as CvMat ptr temp1               ' temporary matrices
  as CvMat ptr temp2
  as CvMat ptr temp3
  as CvMat ptr temp4
  as CvMat ptr temp5
end type

' Creates Kalman filter and sets A, B, Q, R and state to some initial values
declare function cvCreateKalman( dynam_params as long, measure_params as long, _
                                 control_params as long CV_DEFAULT(0) ) as CvKalman ptr

' Releases Kalman filter state
declare sub cvReleaseKalman( kalman as CvKalman ptr ptr )

' Updates Kalman filter by time (predicts future state of the system)
declare function cvKalmanPredict( kalman as CvKalman ptr, control as const CvMat ptr CV_DEFAULT(NULL) ) as const CvMat ptr

' Updates Kalman filter by measurement
' (corrects state of the system and internal matrices)
declare function cvKalmanCorrect( kalman as CvKalman ptr, measurement as const CvMat ptr ) as const CvMat ptr

#define cvKalmanUpdateByTime  cvKalmanPredict
#define cvKalmanUpdateByMeasurement cvKalmanCorrect

end extern ' "C"

